-- main.lua
-- Used for a plugin "ShowZuikaku" to add carrier Zikaku in X-Plane.
-- This Plugin shall be stored in /plugins/ folder in the airplane folder.

setAircraftPanelRendering(false)
set3DRendering(false)
setInteractivity(false)

-- Subwindow setting  ------------------------------------------
local popup = contextWindow {
    name			= "popup",
    position		= { 0, 0, 215, 100},
    noResize		= true,
    visible			= true,
    noBackground    = true,
	layer			= SASL_CW_LAYER_GROWL_NOTIFICATIONS,
    noDecore        = true,
    noMove          = true,
    components		= {infoBoard{position={0, 0, 215, 100},},}
}


-- Initial setting  ---------------------------------------------------
local frigateX = globalPropertyfa("sim/world/boat/x_mtr")           -- OpenGL
local frigateY = globalPropertyfa("sim/world/boat/y_mtr")           -- OpenGL
local frigateZ = globalPropertyfa("sim/world/boat/z_mtr")           -- OpenGL
local boatPsi  = globalPropertyfa("sim/world/boat/heading_deg") -- degree fm Z
local elvRatio = globalPropertyf("sim/flightmodel2/controls/speedbrake_ratio")
local elvOn    = globalPropertyf("sim/cockpit2/controls/speedbrake_ratio")
set(elvOn, 0.0)
local flapDeploy= globalPropertyf("sim/flightmodel2/controls/flap_handle_deploy_ratio")
local Throttle  = globalPropertyf("sim/cockpit2/engine/actuators/throttle_ratio_all")
local Arrest    = globalPropertyi("sim/cockpit/switches/arresting_gear")
local gearDown  = globalPropertyi("sim/cockpit2/controls/gear_handle_down")
local parkBrake = globalPropertyf("sim/cockpit2/controls/parking_brake_ratio")
local tireDef  = globalPropertyfa("sim/flightmodel2/gear/tire_vertical_deflection_mtr")
local groundSpd = globalPropertyf("sim/flightmodel2/position/groundspeed")
local pushBackF = globalPropertyf("sim/flightmodel/forces/faxil_plug_acf")
local pitch = globalPropertyf("sim/cockpit2/controls/yoke_pitch_ratio")
local eRPM=globalPropertyfa("sim/cockpit2/engine/indicators/engine_speed_rpm")
local brakeExt = globalPropertyf("sim/aircraft/controls/acf_speedbrake_ext_time")
local brakeRet = globalPropertyf("sim/aircraft/controls/acf_speedbrake_ret_time")
reloadScn = findCommand("sim/operation/reload_scenery")
goApproach= findCommand("sim/operation/Frigate_Approach")
defineProperty ( "Stage" , 1 )
timerF = sasl.createTimer()

--  #0 Perry data setting
paint0 = { "Perry.dds", "Perry_lit.dds", "Perry_nrm.png"}
function setPerryDref()
	set(globalPropertyf("sim/world/boat/frigate_deck_height_mtr"),    7.0607)
	set(globalPropertyf("sim/world/boat/frigate_ILS_offset_x_mtr"),      0.0)
	set(globalPropertyf("sim/world/boat/frigate_ILS_offset_z_mtr"),48.299999)
end

--  #1 ZUIKAKU data setting
paint1 = { "IJNCarrierZuikaku.png",
           "IJNCarrierZuikaku_night.png",
		   "IJNCarrierZuikaku_normal.png",
		 }                        -- file names must be unique in the plugin 

function setZuiDref()
	set(globalPropertyf("sim/world/boat/frigate_deck_height_mtr"), 14.13)
	set(globalPropertyf("sim/world/boat/frigate_ILS_offset_x_mtr"),  3.0)
	set(globalPropertyf("sim/world/boat/frigate_ILS_offset_z_mtr"),  3.0)
end


-- Main control to activate carrier -----------------------------------------
xppath = sasl.getXPlanePath()
dpath = xppath.."Resources/default scenery/sim objects/dynamic/"
ppath = sasl.getAircraftPath().."/plugins/ShowZuikaku/data/modules/configuration/"
set(Stage, 1)

function goOnZuikaku()
	fileExist = isFileExists( dpath.."(bup)Perry.obj")
	if fileExist == false then
		print(os.rename( dpath.."Perry.obj", dpath.."(bup)Perry.obj"))
		print(os.rename( ppath.."IJNCarrierZuikaku.obj", dpath.."Perry.obj"))
		for i= 1, #paint1 do
			Paint = paint1[i]
			print(os.rename( ppath..Paint, dpath..Paint))
		end
	end
    setZuiDref()
	commandOnce(reloadScn)
   	commandOnce(goApproach)
	set(gearDown, 1)
	Ps = get(boatPsi, 2)
	XX = get(frigateX, 2) - 58.0 * math.sin(Ps * 0.01744)
	YY = get(frigateY, 2) + 16.33
	ZZ = get(frigateZ, 2) + 58.0 * math.cos(Ps * 0.01744)
	lat,lon,alti = sasl.localToWorld( XX, YY, ZZ)
	sasl.placeUserAtLocation ( lat, lon, alti, Ps, 9.9)
	changeToPerry()
	onApproach = 0
	set(Stage, 2)
end

function approachZuikaku()
	fileExist = isFileExists( dpath.."(bup)Perry.obj")
	if fileExist == false then
		print(os.rename( dpath.."Perry.obj", dpath.."(bup)Perry.obj"))
		print(os.rename( ppath.."IJNCarrierZuikaku.obj", dpath.."Perry.obj"))
		for i= 1, #paint1 do
			Paint = paint1[i]
			print(os.rename( ppath..Paint, dpath..Paint))
		end
	end
    setZuiDref()
	commandOnce(reloadScn)
   	commandOnce(goApproach)
	set(gearDown, 1)
	set( parkBrake, 0.0)
	set(flapDeploy, 1.0)
	set(Throttle, 1.0)
	set(Arrest, 1)
	changeToPerry()
	onApproach = 1
	set(Stage, 3)
end

function changeToPerry()
	fileExist = isFileExists( dpath .. "(bup)Perry.obj" )
	if fileExist == true then
	    print(os.rename( dpath.."Perry.obj", ppath.."IJNCarrierZuikaku.obj"))
		for i= 1, #paint1 do
			Paint = paint1[i]
			print(os.rename( dpath .. Paint, ppath .. Paint))
		end
		print(os.rename( dpath.."(bup)Perry.obj", dpath.."Perry.obj"))
	end
	setPerryDref()
end

pitch_ratio = {}
for i = 1, 10 do
    pitch_ratio[i] = 0
end
mean_pitch = 0
THRESHOLD = 0.1

function update()
    if onApproach == 1 then
		if get(Stage) == 3 and get( tireDef, 2) > 0.02  then  -- At touch down
		    if get( groundSpd) > 9.9 then
				set( Stage, 4)
				sasl.startTimer(timerF)
			end
		end

		if get( Stage) == 4 and get( groundSpd) < 12.0  then  -- Aircraft stops
	        set( parkBrake, 0.5)
			set( flapDeploy, 0.0)
	        set( Arrest, 0)
			set( Stage, 2)
			sasl.stopTimer (timerF)
            sasl.resetTimer (timerF)
		end
		
		if get(Stage) == 3 then  
		    set( flapDeploy, 1.0)
		end

		if get(Stage) == 4 then                         -- while applying brake
		    t = sasl.getElapsedSeconds(timerF)
			if t > 1.0 then t = 1.0 end
		    set( pushBackF, 29400.0 * t) 
		else
			set( pushBackF, 0.0)	
		end
	end

	if get( eRPM, 1) < 0.01 then                     -- When engine is shut off
	    mean_pitch = get(pitch)
        for i = 2, 10 do
            mean_pitch = mean_pitch + pitch_ratio[i]
            pitch_ratio[i-1] = pitch_ratio[i]
        end
        pitch_ratio[10] = get(pitch)
        mean_pitch = mean_pitch / 10
			
        if mean_pitch > THRESHOLD then                -- Joystick is pulled 0.1
		    set( pushBackF,  4900.0)
 			set( parkBrake, 0.0)
	    elseif mean_pitch < -THRESHOLD then           -- Joystick is pushed 0.1
			set( pushBackF, -4900.0)
 			set( parkBrake, 0.0)
	    else                                                -- Joystick is idle
			set( parkBrake, 0.5)
	    end
	end
	
	if get( elvRatio) > 1.0 then set( elvRatio, 1.0) end
end

onApproach = 0
topMenu	= sasl.appendMenuItem (PLUGINS_MENU_ID, "Start ZUIKAKU :" )
subMenu	= sasl.createMenu ("", PLUGINS_MENU_ID, topMenu)
subMenu0= sasl.appendMenuItem(subMenu, "on board", goOnZuikaku )
subMenu1= sasl.appendMenuItem(subMenu, "in approach", approachZuikaku )


-- Space key setting ----------------------------------------------------------
set(brakeExt, 8)
set(brakeRet, 8)
function keyHandler(c,Code,shiftDown,ctrlDown,altOptDown,KB_DOWN_EVENT)
	if Code == SASL_VK_SPACE then
		if get( elvRatio) > 0.9 then set( elvOn, 0.0) end
		if get( elvRatio) < 0.1 then set( elvOn, 1.0) end
    end
end
sasl.registerGlobalKeyHandler ( keyHandler )